@@ -2,13 +2,11 @@
 DocumentAI models package.
 """
 
-from .datasets import Dataset
+from ._datasets import Dataset, DatasetStatus
 
 # Enums
-from .enums import (
+from ._enums import (
     ChunkingStrategy,
-    DatasetStatus,
-    JobStatus,
     MimeType,
     ModelProvider,
     ParseStatus,
@@ -16,30 +14,27 @@ from .enums import (
     TableOutputMode,
     TableParsingFormat,
 )
-
+from ._filters import DatasetDataFilter
 
 # Options
-from .options import (
+from ._options import (
     EnrichmentOptions,
     Options,
     PageClassConfig,
     ParsingOptions,
     StructuredExtractionOptions,
 )
-
-# Parse models
-from .parse import ParseRequest
+from ._pagination import PaginatedResult, PaginationDirection
 
 # Results models
-from .results import (
-    PageClass,
-    ParseRequestOptions,
-    ParseResult,
+from ._results import (
     Chunk,
     Figure,
     Page,
+    PageClass,
     PageFragment,
     PageFragmentType,
+    ParseResult,
     Signature,
     StructuredData,
     Table,
@@ -50,7 +45,6 @@ from .results import (
 __all__ = [
     # Enums
     "ChunkingStrategy",
-    "JobStatus",
     "MimeType",
     "ModelProvider",
     "ParseStatus",
@@ -63,11 +57,8 @@ __all__ = [
     "PageClassConfig",
     "ParsingOptions",
     "StructuredExtractionOptions",
-    # Parse models
-    "ParseRequest",
     # Results models
     "PageClass",
-    "ParseRequestOptions",
     "ParseResult",
     "Chunk",
     "Figure",
@@ -82,4 +73,9 @@ __all__ = [
     # Datasets
     "Dataset",
     "DatasetStatus",
+    # Pagination
+    "PaginatedResult",
+    "PaginationDirection",
+    # Filters
+    "DatasetDataFilter",
 ]
